
/**
 * **************************************************
 * **************************************************
 *   _____ _____  ______   _
 *  / ____|  __ \|  ____| | |
 * | |    | |__) | |__    | |    _   _  ___  _ __
 * | |    |  ___/|  __|   | |   | | | |/ _ \| '_ \
 * | |____| |    | |____  | |___| |_| | (_) | | | |
 *  \_____|_|    |______| |______\__, |\___/|_| |_|
 *                                __/ |
 *                               |___/
 * **************************************************
 * **************************************************
 *
 *
 *
 * Les fonctions de entree_sortie permettent de simplifier la communication
 *  avec ce programme ou d'autres programmes par le biais de fichiers.
 * Il est possible d'enregistrer de maniere lisible l'etat de l'echiquier dans un fichier.
 * Il est egalement possible de lire l'etat d'un echiquier a partir de ce type de fichier.
 *
 * Un fichier d'etat est code en ASCII lisible et possede la structure suivante:
 *
 * Joueur 1 (*)   : On indique que l'on va donner les pieces du joueur 0 (pieces blanches), ou 1 (piece noires).
 *                  Le sigle (*) indique que c'est celui ci qui a la main. Il n'est present que derriere l'un des deux joueurs.
 * t0 x0 y0       :
 * t1 x1 y1       :
 * t2 x2 y1       :
 * ...            :
 * ti xi yi       : Le type de la piece i (ti), la coordonnee x de la piece i (xi), la coordonnee y de la piece i (yi)
 * ...            :
 * Joueur 2       : Le joueur oppose
 * t0 x0 y0       :
 * ...            : idem pour la description de ces pieces
 *
 *
 * ============
 *  Par exemple l'etat d'un jeu initial pourra etre le suivant:
 *
 * Joueur 1 (*)
 * 1 0 0
 * 1 7 0
 * 2 1 0
 * 2 6 0
 * 3 2 0
 * 3 5 0
 * 4 3 0
 * 5 4 0
 * 6 0 1
 * 6 1 1
 * 6 2 1
 * 6 3 1
 * 6 4 1
 * 6 5 1
 * 6 6 1
 * 6 7 1
 * Joueur 2
 * 1 0 7
 * 1 7 7
 * 2 1 7
 * 2 6 7
 * 3 2 7
 * 3 5 7
 * 4 4 7
 * 5 3 7
 * 7 0 6
 * 7 1 6
 * 7 2 6
 * 7 3 6
 * 7 4 6
 * 7 5 6
 * 7 6 6
 * 7 7 6
 *
 *
 */


//#Nom1: ZZZ
//#Prenom1: ZZZZ
//#Nom2: ZZZ
//#Prenom2: ZZZZ
//#Groupe: Z



#ifndef ENTREE_SORTIE_H
#define ENTREE_SORTIE_H

#include "echiquier.h"
#include "historique_partie.h"

/**
 * Fonction entree_sortie_est_fichier_accessible:
 * ********************************
 *   Verifie qu'un fichier designee par son chemin est bien accessible (ecriture).
 *
 *   Necessite:
 *     - Un chemin vers un fichier passee sous forme de chaine de caractere non NULL.
 *   Garantie:
 *     - Renvoie valeur 1 si le fichier designee est accessible (peut etre ecrit).
 *     - Renvoie valeur 0 si le fichier n'est pas accessible (ex. mauvais chemin).
 *
 */
int entree_sortie_est_fichier_accessible(const char* filename);

/**
 * Fonction entree_sortie_est_fichier_existe:
 * ********************************
 *   Verifie qu'un fichier designee par son chemin existe (peut etre lue potentiellement).
 *
 *   Necessite:
 *     - Un chemin vers un fichier passee sous forme de chaine de caractere non NULL.
 *   Garantie:
 *     - Renvoie valeur 1 si le fichier designee existe.
 *     - Renvoie valeur 0 si le fichier n'existe pas ou n'est pas accessible.
 *
 */
int entree_sortie_est_fichier_existe(const char* filename);

/**
 * Fonction entree_sortie_ecrit_echiquier_fichier:
 * ********************************
 *   Ecrit de maniere lisible l'etat de l'echiquier dans un fichier.
 *
 *   Necessite:
 *     - Un chemin vers un fichier accessible. (pointeur non NULL)
 *     - Un pointeur constant vers un echiquier valide qui va etre ecrit dans le fichier. (pointeur non NULL)
 *   Garantie:
 *     - Le fichier designee par "filename" contient l'etat de l'echiquier ecrit en ASCII
 *
 */
void entree_sortie_ecrit_fichier_echiquier(const char* filename,const echiquier* echiquier_a_ecrire);



/**
 * Fonction entree_sortie_lit_fichier_echiquier:
 * ********************************
 *   Lit un fichier stockant l'etat d'un echiquier et le stocke dans le parametre "echiquier_a_ecrire".
 *
 *   Necessite:
 *     - Un chemin vers un fichier lisible. (pointeur non NULL)
 *     - Le fichier doit decrire un etat valide d'echiquier.
 *     - Un pointeur non constant vers un echiquier qui va etre mis a jour d'apres le fichier. (pointeur non NULL)
 *   Garantie:
 *     - L'echiquier passe en parametre contient l'etat decrit dans le fichier. Son etat est valide.
 *
 */
void entree_sortie_lit_fichier_echiquier(const char* filename,echiquier* echiquier_a_completer);



/**
 * Fonction entree_sortie_ecrit_fichier_historique:
 * ********************************
 *   Ecrit de maniere lisible l'etat de l'historique dans un fichier.
 *
 *   Necessite:
 *     - Un chemin vers un fichier accessible. (pointeur non NULL)
 *     - Un pointeur constant vers un historique d'echiquier valide qui va etre ecrit dans le fichier. (pointeur non NULL)
 *   Garantie:
 *     - Le fichier designee par "filename" contient l'etat de l'historique ecrit en ASCII
 *
 */
void entree_sortie_ecrit_fichier_historique(const char* filename,const historique_partie* historique_a_ecrire);

/**
 * Fonction entree_sortie_lit_fichier_historique:
 * ********************************
 *   Lit un fichier stockant l'etat de l'historique d'un echiquier et le stocke dans le parametre "historique_a_completer".
 *
 *   Necessite:
 *     - Un chemin vers un fichier lisible. (pointeur non NULL)
 *     - Le fichier doit decrire un etat valide d'historique.
 *     - Un pointeur non constant vers un historique de partie qui va etre mis a jour d'apres le fichier. (pointeur non NULL)
 *   Garantie:
 *     - L'historique passe en parametre contient l'etat decrit dans le fichier. Son etat est valide.
 *
 */
void entree_sortie_lit_fichier_historique(const char* filename,historique_partie* historique_a_completer);

#endif
