
/**
 * **************************************************
 * **************************************************
 *   _____ _____  ______   _
 *  / ____|  __ \|  ____| | |
 * | |    | |__) | |__    | |    _   _  ___  _ __
 * | |    |  ___/|  __|   | |   | | | |/ _ \| '_ \
 * | |____| |    | |____  | |___| |_| | (_) | | | |
 *  \_____|_|    |______| |______\__, |\___/|_| |_|
 *                                __/ |
 *                               |___/
 * **************************************************
 * **************************************************
 *
 *
 * L'historique d'une partie sauvegarde les anciens coups joues sur l'echiquier.
 * L'historique est utilise pour l'application des coups speciaux qui dependent des
 *  coups joues precedemment.
 * L'historique n'a pas connaissance des regles du jeu et n'est la que pour sauvegarder
 *  les deplacements.
 *
 */

//#Nom1: ZZZ
//#Prenom1: ZZZZ
//#Nom2: ZZZ
//#Prenom2: ZZZZ
//#Groupe: Z

#ifndef HISTORIQUE_H
#define HISTORIQUE_H

#include "piece.h"
#include "type_deplacement.h"
#include "echiquier.h"



/**
 * **************************************************
 * **************************************************
 *
 * DECLARATION DES TYPES ET ENUMERATIONS POUR L'HISTORIQUE
 *
 * **************************************************
 * **************************************************
 */


/** La structure deplacement_piece permet de stocker les informations
 *  concernant le deplacement d'une piece. Elle enregistre les coordonnees
 *  de departs et d'arrivees, ainsi que le type de deplacement pour les cas
 *  speciaux.
 */
typedef struct
{
    int x_depart;          //coordonnee x de depart
    int y_depart;          //coordonnee y de depart
    int x_arrivee;         //coordonnee x d'arrivee
    int y_arrivee;         //coordonnee y d'arrivee
    type_deplacement type; //le type de deplacement associe
}deplacement_piece;


/** La structure d'historique d'une partie permet de sauvegarder
 *  les deplacements precedents et le type de deplacement dans les
 *  cas speciaux.
 *  Cet historique est utile pour verifier que certains coups sont
 *  valides (non deplacement du roi et de la tour pour le roque,
 *  dernier coup joue pour la prise en passant, repetition de coups, etc)
 *
 *
 *  L'historique est stocke sous forme d'un tableau de deplacement_piece
 *  de taille predefinie. La variable "nombre_de_coup" indique le nombre
 *  effectif de coups enregistres.
 *
 */
#define MAX_HISTORIQUE 256
typedef struct
{
    deplacement_piece tableau_coup[MAX_HISTORIQUE]; // le tableau contenant l'historique des coups
    echiquier echiquier_initial;  //la configuration de l'echiquier au depart
    int nombre_de_coup;
} historique_partie;



/**
 * **************************************************
 * **************************************************
 *
 * DECLARATION DES FONCTIONS DE L'HISTORIQUE D'UNE PARTIE
 *
 * **************************************************
 * **************************************************
 */


/**
 * Fonction deplacement_invalider:
 * ********************************
 *   Place les parametres d'un deplacement a l'état invalide.
 *    - Coordonnees mises a -1
 *    - Type de deplacement mis a "deplacement_invalide"
 *
 *   Necessite:
 *     - Un pointeur non constant vers une structure deplacement (pointeur non NULL).
 *   Garantie:
 *     - La structure de deplacement est invalide.
 *
 */
void deplacement_invalider(deplacement_piece* deplacement);

/**
 * Fonction deplacement_est_valide:
 * ********************************
 *   Verifie si un deplacement possede des champs valides.
 *   C'est a dire si les coordoonnees de departs et d'arrivees
 *   sont valides, et si le type de deplacement est egalement valide.
 *
 *   Necessite:
 *     - Un pointeur constant vers une structure deplacement (pointeur non NULL).
 *   Garantie:
 *     - Renvoie 1 si les champs du deplacement sont valides.
 *     - Renvoie 0 sinon.
 *
 */
int deplacement_est_valide(const deplacement_piece* deplacement);


/**
 * Fonction historique_initialise:
 * ********************************
 *   Initialise l'historique de la partie.
 *   Toutes les cases du tableau sont places a des configurations invalides
 *   et le nombre de coup est place a 0. L'echiquier de depart est la configuration
 *   par defaut.
 *
 *   Necessite:
 *     - Un pointeur non constant vers un historique de partie (pointeur non NULL).
 *   Garantie:
 *     - L'historique est mis a jour, l'ensemble des deplacements sont invalides et
 *       le nombre de coup vaut 0.
 *       L'echiquier de depart est la configuration par defaut d'un jeu d'echec.
 *       L'historique est une structure valide avec 0 coups joues.
 *
 */
void historique_initialise(historique_partie* historique);

/**
 * Fonction historique_est_valide:
 * ********************************
 *   Initialise l'historique de la partie.
 *   Toutes les cases du tableau sont places a des configurations invalides
 *   et le nombre de coup est place a 0.
 *
 *   Necessite:
 *     - Un pointeur non constant vers un historique de partie (pointeur non NULL).
 *   Garantie:
 *     - L'historique est mis a jour, l'ensemble des deplacements sont invalides et
 *       le nombre de coup vaut 0.
 *       Ne garantie pas que les coups enregistres sont effectivement jouables.
 *
 */
int historique_est_valide(const historique_partie* historique);




/**
 * Fonction historique_ajoute_coup:
 * ********************************
 *   Ajoute un coup a l'historique courant.
 *
 *   Necessite:
 *     - Un pointeur non constant vers un historique de partie valide (pointeur non NULL).
 *     - Des coordonnees de departs et d'arrivees valides.
 *     - Un type de deplacement valide.
 *     - Le nombre de coup est inferieur a MAX_HISTORIQUE-1
 *   Garantie:
 *     - Le deplacement correspondant est ajoute a l'historique.
 *     - Le nombre de coup est incremente de 1.
 *     - L'historique est valide.
 *
 */
void historique_ajoute_coup(historique_partie* historique,int x_depart,int y_depart,int x_arrivee,int y_arrivee,type_deplacement coup);


/**
 * Fonction historique_affiche_debug:
 * ********************************
 *   Affiche l'ensemble "nombre_de_coup" deplacements contenus dans l'historique.
 *   Utile a des fins de debugs.
 *
 *   Necessite:
 *     - Un pointeur constant vers un historique de partie (pointeur non NULL).
 *   Garantie:
 *     - Affiche sur la sortie standard le nombre de coup et les deplacements associes
 *       ainsi que l'etat initial de l'historique.
 *
 */
void historique_affiche_debug(const historique_partie* historique);


/**
 * Fonction historique_recupere_dernier_deplacement_information:
 * ********************************
 *   Renvoie le dernier deplacement de piece jouee (utile pour la prise en passant).
 *
 *   Necessite:
 *     - Un pointeur constant vers un historique de partie (pointeur non NULL).
 *   Garantie:
 *     - Renvoie le dernier deplacement realise (pointeur constant) si le nombre de coup est au moins de 1.
 *     - Renvoie NULL si il s'agit du premier coup.
 */
const deplacement_piece* historique_recupere_dernier_deplacement_information(const historique_partie* historique);





/**
 * Fonction historique_deplacement_existe:
 * ********************************
 *   Recherche si il existe un deplacement au depart des coordoonnees indiquees
 *   (utile pour le cas du roque).
 *
 *   Necessite:
 *     - Un pointeur constant vers un historique de partie (pointeur non NULL).
 *     - Des coordonnees de departs valides.
 *   Garantie:
 *     - Renvoie 1 si il existe un deplacement au depart des coordoonees indiquees.
 *     - Renvoie 0 si aucun deplacement au depart des coordoonnees n'est trouve.
 */
//implementer int historique_deplacement_existe(const historique_partie* historique,int x_depart,int y_depart);

#endif
